# Copyright 2015 Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Fastfood package attributes and metadata."""

__all__ = (
    '__title__',
    '__summary__',
    '__author__',
    '__email__',
    '__version__',
    '__license__',
    '__copyright__',
    '__url__',
)

__title__ = 'fastfood'
__summary__ = 'Chef Cookbook Wizardry'
__author__ = 'Rackers'
__email__ = 'samuel.stavinoha@rackspace.com'
__version__ = '1.1.0'
__keywords__ = ['chef', 'cookbook', 'templating', 'generation']
__license__ = 'Apache License, Version 2.0'
__copyright__ = 'Copyright Rackspace US, Inc. 2015'
__url__ = 'https://github.com/rackerlabs/fastfood'
